#include "xmlwrap.h"

#include "../../misc/log_time.h"

XMLNode::XMLNode(struct _xmlNode* cur) {
  odlog(4)<<"XMLNode "<<this<<": contructor: from struct _xmlNode"<<endl;
  instance=cur; allocated=false;
}

XMLNode::XMLNode(const std::string& name) {
  odlog(4)<<"XMLNode "<<this<<": contructor: from name: "<<name<<endl;
  instance=xmlNewNode(NULL,(const xmlChar*)(name.c_str()));
  allocated=true;
}

XMLNode::XMLNode(XMLNode& parent,const std::string& name) {
  odlog(4)<<"XMLNode "<<this<<": contructor: linked and name: "<<name<<endl;
  instance=xmlNewNode(NULL,(const xmlChar*)(name.c_str()));
  allocated=true;
  parent.linkNode(*this);
}

XMLNode::XMLNode(const XMLNode& node) {
  odlog(4)<<"XMLNode "<<this<<": contructor: from node"<<endl;
  instance=NULL;
  if(node.instance==NULL) return;
  instance=xmlCopyNode(node.instance,1); 
  allocated=true;
}

void XMLNode::operator=(const XMLNode& node) {
  if(instance != NULL) {
    if(allocated) xmlFreeNode(instance);
    instance=NULL;
  };
  if(node.instance==NULL) return;
  instance=xmlCopyNode(node.instance,1); 
  allocated=true;
}

XMLNode::~XMLNode(void) {
  odlog(4)<<"XMLNode "<<this<<": destructor"<<endl;
  if(instance == NULL) return;
  odlog(4)<<"XMLNode "<<this<<": destructor: with instance: "<<allocated<<endl;
  if(allocated) xmlFreeNode(instance);
  instance=NULL;
}

void XMLNode::addNode(const XMLNode& node) {
  odlog(4)<<"XMLNode "<<this<<": addNode"<<endl;
  if(instance == NULL) return;
  if(node.instance == NULL) return;
  struct _xmlNode* cur = xmlCopyNode(node.instance,1); 
  if(cur == NULL) return;
  xmlAddChild(instance,cur);
}

void XMLNode::linkNode(XMLNode& node) {
  odlog(4)<<"XMLNode "<<this<<": linkNode"<<endl;
  if(instance == NULL) return;
  if(node.instance == NULL) return;
  struct _xmlNode* cur = xmlAddChild(instance,node.instance);
  if(cur != NULL) {
    node.instance=cur;
    node.allocated=false;
  };
}

XMLNode* XMLNode::getNode(int n) const {
  odlog(4)<<"XMLNode "<<this<<": getNode: "<<n<<endl;
  if(instance == NULL) return NULL;
  struct _xmlNode* cur = instance->children;
  for(;(cur != NULL) && (n>0);cur=cur->next,n--) { };
  if(cur == NULL) return NULL;
  return new XMLNode(cur);
}

XMLNode* XMLNode::getNode(const std::string& name) const {
  odlog(4)<<"XMLNode "<<this<<": getNode: "<<name<<endl;
  if(instance == NULL) return NULL;
  struct _xmlNode* cur = instance->children;
  for(;cur != NULL;cur=cur->next) {
    if(cur->name == NULL) continue;
    if(name == (const char*)(cur->name)) break;
  };
  if(cur == NULL) return NULL;
  return new XMLNode(cur);
}

XMLNode* XMLNode::getNode(const std::string& name,int n) const {
  odlog(4)<<"XMLNode "<<this<<": getNode: "<<name<<" "<<n<<endl;
  if(instance == NULL) return NULL;
  struct _xmlNode* cur = instance->children;
  for(;cur != NULL;cur=cur->next) {
    if(cur->name == NULL) continue;
    if(name == (const char*)(cur->name)) {
      if((n--) == 0) break;
    };
  };
  if(cur == NULL) return NULL;
  return new XMLNode(cur);
}

int XMLNode::nodes(void) const {
  odlog(4)<<"XMLNode "<<this<<": nodes"<<endl;
  if(instance == NULL) return 0;
  struct _xmlNode* cur = instance->children;
  int n = 0;
  for(;cur != NULL;cur=cur->next) { };
  odlog(4)<<"XMLNode "<<this<<": nodes: "<<n<<endl;
  return n;
}

std::string XMLNode::name(void) const {
  if(instance == NULL) return "";
  if(instance->name == NULL) return "";
  return std::string((const char*)(instance->name));
}

void XMLNode::name(const std::string& s) {
  odlog(4)<<"XMLNode "<<this<<": name: "<<s<<endl;
  if(instance == NULL) {
    odlog(4)<<"XMLNode "<<this<<": name: creating new instance"<<endl;
    instance=xmlNewNode(NULL,(const xmlChar*)(s.c_str()));
    allocated=true;
    return;
  };
  xmlNodeSetName(instance,(const xmlChar*)(s.c_str()));
}

std::string XMLNode::content(void) const {
  if(instance == NULL) return "";
  xmlBufferPtr buf = xmlBufferCreate();
  if(buf == NULL) return "";
  if(xmlNodeBufGetContent(buf,instance) != 0) {
    xmlBufferFree(buf); return "";
  };
  std::string s((const char*)xmlBufferContent(buf));
  return s;
}
  
void XMLNode::content(const std::string& s) {
  odlog(4)<<"XMLNode "<<this<<": content: "<<s<<endl;
  if(instance == NULL) return;
  xmlNodeSetContent(instance,(const xmlChar*)(s.c_str()));
}

std::string XMLNode::get(void) const {
  odlog(4)<<"XMLNode "<<this<<": get"<<endl;
  if(instance == NULL) return "";
  xmlBufferPtr buf = xmlBufferCreate();
  if(buf == NULL) return "";
  if(xmlNodeDump(buf,NULL,instance,0,0) == -1) return "";
  std::string s((const char*)xmlBufferContent(buf));
  odlog(4)<<"XMLNode "<<this<<": get: "<<s<<endl;
  return s;
}

void XMLNode::set(const std::string& s) {
  odlog(4)<<"XMLNode "<<this<<": set: "<<s<<endl;
  if(instance != NULL) {
    if(allocated) xmlFreeNode(instance);
    instance=NULL;
  };
  xmlDocPtr doc = xmlParseMemory(s.c_str(),s.length());
  xmlNodePtr cur = xmlDocGetRootElement(doc);
  instance=xmlCopyNode(cur,1);
  xmlFreeDoc(doc);
  allocated=true;
}

