package se.umu.jobsubmission.impl.broker;

import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

import se.umu.jobsubmission.FileTransfer;
import se.umu.jobsubmission.JobPreferencesType;
import se.umu.jobsubmission.SchedulingHintsType;
import se.umu.jobsubmission.impl.datamanager.DataHelper;
import se.umu.jobsubmission.util.SelectorUtil;
import se.umu.jobsubmission.util.ThreadPool;
import se.umu.jobsubmission.util.Worker;


import org.ggf.schemas.jsdl._2005._06.jsdl.JobDefinition_Type;

public class EarliestCompletionSelector implements Selector {
    
	private static Logger logger = Logger.getLogger(
			EarliestCompletionSelector.class);
	
	private Predictor startTimePredictor; 
	private Predictor runTimePredictor; 
	private Predictor stageOutTimePredictor; 
	private Predictor stageInTimePredictor; 
	private int numberOfThreads; 
	
	public EarliestCompletionSelector(Predictor startTimePredictor, 
			int numberOfThreads) {
		this.startTimePredictor = startTimePredictor; 
		this.runTimePredictor = new RunTimePredictor(); 
		this.stageOutTimePredictor = new StageOutTimePredictor(); 
		this.stageInTimePredictor = new StageInTimePredictor(); 
		this.numberOfThreads = numberOfThreads; 
	}
	
	public List orderResources(List resources, 
			JobDefinition_Type jobDescription, 
			JobPreferencesType preferences) {
		
		logger.debug("Reordering resources");     	
		
		int startTime = 0; 
		int runTime = 0; 
		int stageOutTime = 0; 
		Object resource = null; 
		String resourceName = null; 
		
		List completionTimes = new LinkedList(); 
		
		// Check if user specified input staging time in job preferences. 
		int stageInTime = -1;    
		try {    	    	    	  
			SchedulingHintsType hints = preferences.getSchedulingHints(); 
			FileTransfer transfer = hints.getFileTransfers();
			Integer time = transfer.getStageInTime(); 
			stageInTime = time.intValue(); 
		} catch (Exception e) {
			logger.debug("Could not determine stage in" +
			" time from job preferences");     			    		    							
		}
		// No stage in prediction, get size of all input files for later use. 
		Map fileSizes = null; 
		if(stageInTime < 0) {
			try {
				List fileNames = DataHelper.getInputFiles(jobDescription); 
				fileSizes = DataHelper.getFileSizes(fileNames); 
			} catch (Exception e) {
				logger.warn("Can not determine size of job input"); 
			}
		}    	
		
		// Predict start time for each resource and save times in a list. 
		ThreadPool pool = new ThreadPool(this.numberOfThreads);
		List workers = new LinkedList(); 
		for(int i = 0; i < resources.size(); i++) {
			
			Worker w = new EarliestCompletionWorker(resources.get(i), 
					jobDescription, 
					preferences, 
					fileSizes, 
					this.stageInTimePredictor, 
					this.startTimePredictor, 
					this.runTimePredictor, 
					this.stageOutTimePredictor); 
			workers.add(w); 
		}
		
		List results = pool.work(workers); 
		
		List resourceResults = new LinkedList(); 
		List rankResults = new LinkedList(); 
		for(int i = 0; i < results.size(); i++) {
			RankedResource r = (RankedResource) results.get(i); 
			resourceResults.add(r.getResourceInformation()); 
			rankResults.add(new Integer(r.getTime())); 
		}
			
		List reordered =  SelectorUtil.reorderResources(resourceResults, 
				rankResults); 
		pool.shutdown(); 
		
		return reordered; 
			
	}
}
