/*
 * Created on Sep 30, 2005
 *
 * Translates a JSDL job description to (WS) RSL. 
 *
 */
package se.umu.jobsubmission.impl.dispatcher;

import java.util.List;
import java.util.Vector;

import org.apache.axis.message.MessageElement;
import org.apache.axis.types.NonNegativeInteger;
import org.apache.axis.types.PositiveInteger;
import org.apache.log4j.Logger;
import org.ggf.schemas.jsdl._2005._06.jsdl.DataStaging_Type;
import org.ggf.schemas.jsdl._2005._06.jsdl.JobDefinition_Type;
import org.ggf.schemas.jsdl._2005._06.jsdl.JobDescription_Type;
import org.ggf.schemas.jsdl._2005._06.jsdl.JobIdentification_Type;
import org.ggf.schemas.jsdl._2005._06.jsdl.RangeValue_Type;
import org.ggf.schemas.jsdl._2005._06.jsdl.Resources_Type;
import org.ggf.schemas.jsdl._2005._06.jsdl.SourceTarget_Type;
import org.ggf.schemas.jsdl._2005._06.jsdl_posix.Argument_Type;
import org.ggf.schemas.jsdl._2005._06.jsdl_posix.DirectoryName_Type;
import org.ggf.schemas.jsdl._2005._06.jsdl_posix.Environment_Type;
import org.ggf.schemas.jsdl._2005._06.jsdl_posix.FileName_Type;
import org.ggf.schemas.jsdl._2005._06.jsdl_posix.POSIXApplication_Type;
import org.globus.exec.generated.JobDescriptionType;
import org.globus.exec.generated.NameValuePairType;
import org.globus.rft.generated.TransferRequestType;
import org.globus.rft.generated.TransferType;
import org.globus.wsrf.encoding.ObjectDeserializer;

import se.umu.jobsubmission.util.JobDescriptionTranslator;
/**
 * @author tordsson
 *
 * TODO Add a proper description of this class. 
 */
public class JSDL2RSL implements JobDescriptionTranslator {
	
	
	private static Logger logger = Logger.getLogger(JSDL2RSL.class); 
	
    public Object translate(Object jobDesc) throws Exception {
    	
    	JobDefinition_Type jobDefinition = (JobDefinition_Type) jobDesc; 
    	JobDescription_Type jsdl = jobDefinition.getJobDescription(); 
    	Resources_Type resource = jsdl.getResources(); 
    	
    	JobDescriptionType rsl = new JobDescriptionType(); 
    	POSIXApplication_Type posix = null; 
    	
    	try {
    		MessageElement any[] = jsdl.getApplication().get_any(); 
    		 posix = (POSIXApplication_Type) 
				ObjectDeserializer.toObject(any[0], POSIXApplication_Type.class); 
    	} catch (Exception e) {
    		logger.error("Invalid job descriptions, lacks POSIXAppl. element"); 
    		throw new Exception("Job description lacks POSIXAppl. element");
    	}
    	
    	// executable
    	FileName_Type executable =  posix.getExecutable(); 
    	if(executable != null) {
    		rsl.setExecutable(executable.get_value()); 
    	} 
    	else {
    		logger.error("Job description does not specify executable"); 
    		throw new Exception("Job description does not specify executable"); 
    	}
    	
    	// arguments
    	Argument_Type arguments[] = posix.getArgument(); 
    	if(arguments != null) {
    		String args[] = new String[arguments.length]; 
    		for(int i = 0; i < arguments.length; i++) {
    			args[i] =  arguments[i].get_value().toString(); 
    		}
    		rsl.setArgument(args); 
    	}
    	
    	// stdin 
    	FileName_Type stdin = posix.getInput(); 
    	if(stdin != null) {
    		rsl.setStdin(stdin.get_value()); 
    	}
    	
    	// stdout
    	FileName_Type stdout = posix.getOutput(); 
    	if(stdout != null) {
    		rsl.setStdout(stdout.get_value());
    	}
    	
    	// stderr
    	FileName_Type stderr = posix.getError(); 
    	if(stderr != null) {
    		rsl.setStderr(stderr.get_value()); 
    	}
    	
    	// directory
    	DirectoryName_Type directory =  posix.getWorkingDirectory(); 
    	if(directory != null) {
    		rsl.setDirectory(directory.get_value()); 
    	}
    	
    	// environment
    	Environment_Type environment[] =  posix.getEnvironment(); 
    	if(environment != null) {
    		
    		NameValuePairType env[] = new NameValuePairType[environment.length]; 
    		for(int i = 0; i < environment.length; i++) {
    			env[i] = new NameValuePairType();  
    			env[i].setName(environment[i].getName().toString()); 
    			env[i].setValue(environment[i].get_value()); 
    		}
    		rsl.setEnvironment(env); 
    	}
    	
		
		// Project
    	JobIdentification_Type jobIdentification = jsdl.getJobIdentification(); 
    	if(jobIdentification != null) {
    		String project = jobIdentification.getJobProject(0); 
    		if(project != null) {
    			rsl.setProject(project); 
    		}
    	}
    	
    	// fileStageIn / fileStageOut
    	DataStaging_Type dataStaging[] = jsdl.getDataStaging(); 
    	if(dataStaging != null) {
    		
    		List inputFiles = new Vector(); 
    		List outputFiles = new Vector(); 
    		TransferType transfer; 
    		for(int i = 0; i < dataStaging.length; i++) {
    			
    			transfer = new TransferType(); 
    			SourceTarget_Type source = dataStaging[i].getSource(); 
    			SourceTarget_Type target = dataStaging[i].getTarget(); 
    			String fileName = dataStaging[i].getFileName(); 
    			
    			// input file: 
    			if(target == null) {
    				transfer.setDestinationUrl(fileName); 
    				transfer.setSourceUrl(source.getURI().toString()); 
    				inputFiles.add(transfer); 
    			} 
    			// output file 
    			if(source == null) {
    				transfer.setSourceUrl(fileName); 
    				transfer.setDestinationUrl(target.getURI().toString()); 
    				outputFiles.add(transfer); 
    			}
    		}
    		// Add inputfiles, if any, to rsl. 
    		if(inputFiles.size() > 0) {
    			TransferRequestType inputRequest = new TransferRequestType(); 
    			TransferType input[] = new TransferType[inputFiles.size()]; 
    			for(int i = 0; i < inputFiles.size(); i++) {
    				input[i] = (TransferType) inputFiles.get(i); 
    			}
    			inputRequest.setTransfer(input); 
    			rsl.setFileStageIn(inputRequest); 
    		} // Add any output files...
    		if(outputFiles.size() > 0) {
    			TransferRequestType outputRequest = new TransferRequestType(); 
    			/*
    			TransferType output[] = (TransferType[]) outputFiles.toArray(
    					new TransferType[1]); 
    			*/
    			TransferType output[] = new TransferType[outputFiles.size()]; 
    			for(int i = 0; i < outputFiles.size(); i++) {
    				output[i] = (TransferType) outputFiles.get(i); 
    			}
    			outputRequest.setTransfer(output); 
    			rsl.setFileStageOut(outputRequest); 
    		}
    	}
    	
    	if(resource != null) {
    		
    		// count
    		RangeValue_Type count = resource.getTotalCPUCount();
    		if(count != null) {
    			int countValue = (int) count.getExact(0).get_value();
    			PositiveInteger countInt = new PositiveInteger("" + countValue); 
    			rsl.setCount(countInt); 
    		}
    		
    		// wallTime/cpuTime
    		// TODO: Separate these cases and handle each properly. 
    		RangeValue_Type time = resource.getIndividualCPUTime(); 
    		if(time != null) {
    			long timeValue = (long) time.getExact(0).get_value(); 
    			// Convert back from seconds to minutes...
    			rsl.setMaxWallTime(new Long(timeValue / 60)); 
    		}
    		
    		// memory
    		RangeValue_Type memory = resource.getIndividualPhysicalMemory(); 
    		if(memory != null) {
    			long memoryValue = (long) memory.getExact(0).get_value(); 
    			NonNegativeInteger memoryInt = new NonNegativeInteger("" 
    					+ memoryValue); 
    			rsl.setMaxMemory(memoryInt); 
    		}
    	}
    	
    	return rsl; 
    }
	
}
