package se.umu.jobsubmission.impl.dispatcher; 

import java.util.LinkedList;
import java.util.List;
import java.util.Vector;

import org.apache.axis.message.MessageElement;
import org.apache.axis.types.NormalizedString;
import org.apache.axis.types.URI;
import org.apache.log4j.Logger;
import org.ggf.schemas.jsdl._2005._06.jsdl.Application_Type;
import org.ggf.schemas.jsdl._2005._06.jsdl.CandidateHosts_Type;
import org.ggf.schemas.jsdl._2005._06.jsdl.DataStaging_Type;
import org.ggf.schemas.jsdl._2005._06.jsdl.Exact_Type;
import org.ggf.schemas.jsdl._2005._06.jsdl.JobDefinition_Type;
import org.ggf.schemas.jsdl._2005._06.jsdl.JobDescription_Type;
import org.ggf.schemas.jsdl._2005._06.jsdl.JobIdentification_Type;
import org.ggf.schemas.jsdl._2005._06.jsdl.RangeValue_Type;
import org.ggf.schemas.jsdl._2005._06.jsdl.Resources_Type;
import org.ggf.schemas.jsdl._2005._06.jsdl.SourceTarget_Type;
import org.ggf.schemas.jsdl._2005._06.jsdl_posix.Argument_Type;
import org.ggf.schemas.jsdl._2005._06.jsdl_posix.Environment_Type;
import org.ggf.schemas.jsdl._2005._06.jsdl_posix.FileName_Type;
import org.ggf.schemas.jsdl._2005._06.jsdl_posix.POSIXApplication_Type;
import org.globus.rsl.AbstractRslNode;
import org.globus.rsl.NameOpValue;
import org.globus.rsl.RslNode;
import org.globus.rsl.Value;
import org.globus.wsrf.encoding.ObjectDeserializer;

import se.umu.jobsubmission.util.JobDescriptionTranslator;

/** 
 * This class translates JSDL job descriptions to xRSL 
 * for submission to ARC resources. 
 */

public class jsdl2xrsl implements JobDescriptionTranslator {

    private Logger logger = Logger.getLogger(jsdl2xrsl.class); 

    public void translateJobIdentification(AbstractRslNode xrsl, 
    		JobDescription_Type jsdl) throws Exception {
    	
    	JobIdentification_Type jobID =  jsdl.getJobIdentification(); 
    	if(jobID != null) {
    		String jobName = jobID.getJobName(); 
    		if(jobName != null) {
    			  NameOpValue nov = new NameOpValue("jobName", NameOpValue.EQ, 
    			  		jobName); 
    			  xrsl.add(nov); 
    		}
    	}
		// Project
    	JobIdentification_Type jobIdentification = jsdl.getJobIdentification(); 
    	if(jobIdentification != null) {
    		String project = jobIdentification.getJobProject(0); 
    		if(project != null) {
    			NameOpValue nov = new NameOpValue("project", NameOpValue.EQ, 
    					project); 
    			xrsl.add(nov); 
    		}
    	}
    	
    }
    
    public void translateResource(AbstractRslNode xrsl, 
    		JobDescription_Type jsdl) throws Exception {
    	
       	// Resource contains cluster, count, memory, disk, etc. 
    	Resources_Type resource = jsdl.getResources(); 
    	if(resource != null) {
    		
    		CandidateHosts_Type host = resource.getCandidateHosts(); 
    		if(host != null) {
    			String hostName = host.getHostName(0); 
    			NameOpValue nov = new NameOpValue("cluster", NameOpValue.EQ, 
    					hostName); 
    			xrsl.add(nov); 
    		}
    		
    		RangeValue_Type totalCPUs = resource.getTotalCPUCount();
    		if(totalCPUs != null) {
    			Exact_Type exact[] = totalCPUs.getExact(); 
    			double d = exact[0].get_value(); 
    			NameOpValue nov = new NameOpValue("count", NameOpValue.EQ, 
    					new String("" + (int)d)); 
    			xrsl.add(nov); 
    		}
    		
    		RangeValue_Type memory = resource.getIndividualPhysicalMemory(); 
    		if(memory != null) {
    			Exact_Type exact[] = memory.getExact(); 
    			double d = exact[0].get_value(); 
    			NameOpValue nov = new NameOpValue("memory", NameOpValue.EQ, 
    					new String("" + (int)d)); 
    			xrsl.add(nov);     			
    		}
    		
    		RangeValue_Type diskSpace =  resource.getIndividualDiskSpace(); 
    		if(diskSpace != null) {
    			Exact_Type exact[] = diskSpace.getExact(); 
    			double d = exact[0].get_value(); 
    			NameOpValue nov = new NameOpValue("disk", NameOpValue.EQ, 
    					new String("" + (int)d)); 
    			xrsl.add(nov);     			
    		}
    		
    		RangeValue_Type CPUTime =  resource.getIndividualCPUTime(); 
    		if(CPUTime != null) {
    			Exact_Type exact[] = CPUTime.getExact(); 
    			double d = exact[0].get_value(); 
    			NameOpValue nov = new NameOpValue("cpuTime", NameOpValue.EQ, 
    					new String("" + (((int)d)/60))); 
    			xrsl.add(nov);     			
    		}
    	}
    	
    }
    
    public void translateApplication(AbstractRslNode xrsl, 
    		JobDescription_Type jsdl) throws Exception {
    	
    	Application_Type application = jsdl.getApplication(); 
    	if(application != null) {
    		
    		// POSIXApplication (an extension to Application contains 
    		// executable, arguments, std{in,out,err} etc.
    		MessageElement any[] = application.get_any(); 
    		POSIXApplication_Type posix = (POSIXApplication_Type)
				ObjectDeserializer.toObject(any[0], POSIXApplication_Type.class); 
    		if(posix == null) {
    			// Big trouble...
    			throw new Exception("Job translation failed. no executable specified"); 
    		}
    		else {
    			
    			FileName_Type executable = posix.getExecutable(); 
    			if(executable != null) {
    				String name = executable.get_value(); 
    				NameOpValue nov = new NameOpValue("executable", NameOpValue.EQ, 
    						name); 
    				xrsl.add(nov); 
    			}
    			else 
    				throw new Exception("Job translation failed. no executable specified"); 
    			
    			
    			FileName_Type input = posix.getInput(); 
    			if(input != null) {
    				String name = input.get_value(); 
    				NameOpValue nov = new NameOpValue("stdin", NameOpValue.EQ, 
    						name); 
    				xrsl.add(nov); 
    			}
    			
    			FileName_Type output = posix.getOutput(); 
    			if(output != null) {
    				String name = output.get_value(); 
    				NameOpValue nov = new NameOpValue("stdout", NameOpValue.EQ, 
    						name); 
    				xrsl.add(nov); 
    			}
    			
    			FileName_Type error = posix.getError(); 
    			if(error != null) {
    				String name = error.get_value(); 
    				NameOpValue nov = new NameOpValue("stderr", NameOpValue.EQ, 
    						name); 
    				xrsl.add(nov); 
    			}
    			
    			// arguments, environment. 
    			Argument_Type argument[] = posix.getArgument(); 
    			if(argument != null) {
    				int size = argument.length; 
    				String argumentValues[] = new String[size]; 
    				for(int i = 0; i < size; i++) {
    					NormalizedString name = argument[i].get_value(); 
    					argumentValues[i] = name.toString();
    				}
    				NameOpValue nov = new NameOpValue("arguments", 
    						NameOpValue.EQ, 
							argumentValues); 
    				xrsl.add(nov); 
    			}
    			
    			Environment_Type environment[] = posix.getEnvironment(); 
    			if(environment != null) {
    				int size = environment.length; 
    				logger.debug("Translationg " + size + " environments"); 
    				List allEnvironments = new LinkedList(); 
    				NameOpValue nov = new NameOpValue("environment", NameOpValue.EQ); 
    				for(int i = 0; i < size; i++) {
    					List env = new LinkedList(); 
    					Value envName = new Value(environment[i].getName().toString()); 
    					Value envValue = new Value(environment[i].get_value()); 
    					env.add(envName); 
    					env.add(envValue); 
    					nov.add(env); 
    				}
    				xrsl.add(nov); 
    			}
    			else {
    				logger.debug("No environment specified"); 
    			}
    			
    		}
    	} 
    	else {
    		// This means big trouble, as we have not defined an executable...
    		throw new Exception("Job description must specify \"Executable\""); 
    	}
    }
    
    private void addFile(AbstractRslNode xrslRoot, 
            String relationName, 
            String localFileName, 
            String remoteFileName) {

        if(localFileName == null || 
                remoteFileName == null) 
            return; 

        List newFile = new Vector(); 
        newFile.add(new Value(localFileName)); 
        newFile.add(new Value(remoteFileName)); 
        
        logger.debug("Adding: " + newFile + " to relation " + relationName); 

        NameOpValue fileRelation = xrslRoot.getParam(relationName); 

        if(fileRelation == null) {
            fileRelation = new NameOpValue(relationName, NameOpValue.EQ); 
            xrslRoot.add(fileRelation); 
        }
        fileRelation.add(newFile); 

    }
    
	
    public Object translate(Object jobDesc) throws Exception {
    	
    	
    	logger.debug("Translation description back to xRSL"); 
    	
    	JobDefinition_Type jobDefinition = (JobDefinition_Type) jobDesc; 
    	JobDescription_Type jsdl = jobDefinition.getJobDescription(); 
    	
    	AbstractRslNode xrsl = new RslNode(AbstractRslNode.AND); 
    	
    	this.translateJobIdentification(xrsl, jsdl); 
    	

    
    	// Translate attributes related to resource requirements
    	// e.g., diskspace, memory, count. 
    	this.translateResource(xrsl, jsdl); 
    	
 
    	// Translate attributes related to job configuration 
    	// e.g., executable, stdin, stdout, arguments, environment. 
    	this.translateApplication(xrsl, jsdl); 
 
    	
    	DataStaging_Type dataStaging[] = jsdl.getDataStaging(); 
    	if(dataStaging != null) {
    		int size = dataStaging.length; 
    		for(int i = 0; i < size; i++) {
    			SourceTarget_Type source = dataStaging[i].getSource(); 
    			SourceTarget_Type target = dataStaging[i].getTarget(); 
    			String fileName = dataStaging[i].getFileName(); 
    			
    			// Is this stage-in or stage out? 
    			// target == null -> input file (stage from source to fileName)
    			// source == null -> output file (stage from target to fileName)
    			
    			if(source == null && target == null) {
    				logger.error("Incorrect specified job file"); 
    				throw new Exception("Incorrect specified job file"); 
    			}
    			
    			// Input file
    			String name = null; 
    			if(source != null) {
    				URI uri = source.getURI(); 
    				name = uri.toString(); 
    				logger.debug("File " + fileName + " will be stored at " 
    						+ name);
    				this.addFile(xrsl, "inputfiles", fileName, name); 
    			}
    			// Output file 
    			if(target != null) {
    				URI uri = target.getURI(); 
    				if(uri == null) {
    					name = ""; 
    				}
    				else {
    					name = uri.toString(); 
    				}
    				logger.debug("File " + fileName + " will be stored at " 
    						+ name);
    				this.addFile(xrsl, "outputfiles", fileName, name); 
    				
    			}
    		}
    		
    	}
    	
    	
    	
    	
    	
    	return xrsl; 
    }

}
