/*
 * Created on Oct 14, 2005
 *
 */
package se.umu.jobsubmission.impl.informationfinder;

import org.apache.axis.message.MessageElement;
import org.apache.log4j.Logger;
import org.globus.axis.util.Util;
import org.globus.gsi.GlobusCredential;
import org.globus.mds.glue.GLUECERPType;
import org.globus.mds.glue.StateType;
import org.globus.mds.glue.batchprovider.SchedulerType;
import org.globus.mds.glue.batchprovider.QueueType;
import org.globus.mds.glue.batchprovider.IntUnlimitedUnknownType;
import org.globus.mds.glue.InfoType;
import org.globus.mds.glue.ComputingElementType;
import org.globus.wsrf.encoding.ObjectDeserializer;
import org.globus.wsrf.utils.AnyHelper;
import org.oasis.wsrf.properties.QueryResourcePropertiesResponse;
import org.oasis.wsrf.properties.QueryResourceProperties_Element;
import org.oasis.wsrf.properties.QueryResourceProperties_PortType;

import se.umu.jobsubmission.util.QueryUtils;
import se.umu.wsag.util.JobSubmissionUtils;

import org.globus.wsrf.client.ServiceURL;


/**
 * @author tordsson
 * Based on GT4 wsrf-query client, used to retrieve information 
 * about a single GT4 resource. 
 * 
 * TODO: This must be rewritten once a correctly installed 
 * information system is in place at the toggle machines. 
 * 
 */
public class GT4InformationFetcher implements InformationFetcher {
	
	private static Logger logger = Logger.getLogger(GT4InformationFetcher.class); 
	
	// Test this one...
	public String translateURL(String url) {
		
		if(url.startsWith("http")) {
			return url; 
		}
		
		String newURL =  ServiceURL.getDefaultProtocol() 
			+ "://" 
			+ url 
			+ ":"
			+ ServiceURL.getDefaultPort()
			+ "/wsrf/services/DefaultIndexService"; 
		
		return newURL; 
		
	}
	
	/**
	 * Adds some of the information from the schedulerObject to the
	 * Glue information (at appropiate locations). 
	 * @param glueInfo
	 * @param schedulerInfo
	 */
	public void updateGlue(GLUECERPType glueInfo, SchedulerType schedulerInfo) {
		
		ComputingElementType ce[] = glueInfo.getComputingElement(); 
		if(ce != null) {
			StateType state = ce[0].getState(); 
			
			if(state != null) {
				
				// totaljobs
				String totalJobs = schedulerInfo.getQueue(0).getTotalJobs().toString(); 
				if("unknown".equals(totalJobs)) {
					state.setTotalJobs(0); // Use some other value? 
				}
				else if("unlimited".equals(totalJobs)) {
					state.setTotalJobs(Integer.MAX_VALUE); 
				} else {
					state.setTotalJobs(new Integer(totalJobs).intValue()); 
				}
			} 
			else {
				logger.warn("\"CE:State\" is null"); 
			}
		} else {
			logger.warn("\"CE\" is null"); 
		}
		
		// Value of most of these attributes can be 
		// "17", "unlimited" or "unknown", which make 
		// them kind of messy to handle. 
		
	
		
		/* FIXME: 
		 This is the old and currently not supported format of SchedulerType 
		 <scheduler xmlns="http://mds.globus.org/batchproviders/2004/09" 
		 xmlns:ns12="http://mds.globus.org/batchproviders/2004/09">
		 <Info GRAMVersion="4.0.1" HostName="toggle01.hpc2n.umu.se" 
		 LRMSType="PBS" LRMSVersion="2.0.0p2" TotalCPUs="8"/>
		 
		 <Queue name="batch">
		 <totalnodes>1</totalnodes>
		 <freenodes>1</freenodes>
		 <maxCount>1</maxCount>
		 <maxTotalMemory>unlimited</maxTotalMemory>
		 <maxSingleMemory>unlimited</maxSingleMemory>
		 <maxCPUtime>unlimited</maxCPUtime>
		 <maxtime>7200</maxtime>
		 <maxReqNodes>unlimited</maxReqNodes>
		 <runningJobs>3</runningJobs>
		 <maxRunningJobs>unlimited</maxRunningJobs>
		 <maxJobsInQueue>unlimited</maxJobsInQueue>
		 <whenActive>unknown</whenActive>
		 <dispatchType>batch</dispatchType>
		 <totalJobs>3</totalJobs>
		 <status>enabled</status>
		 */
		
		// totalCPUs
		// This is already set in the state.freeCPUs, but 
		// the number is incorrect @ toggle
		// Furtermore, the schemas has changed and 
		// SchedulerType:Info is removed -> no way to access correct 
		// number of CPUs. 
		
		// freeCPUs
		// This is already set in the state.freeCPUs, but 
		// the number is incorrect @ toggle...
		
		// TODO: 
		// The following make up some rather useful policy info
		// and should be appended to GLUE:CE:Policy
		/*
	      <maxTotalMemory>unlimited</maxTotalMemory>
	      <maxSingleMemory>unlimited</maxSingleMemory>
	      <maxCPUtime>unlimited</maxCPUtime>
	      <maxtime>7200</maxtime>
	      <maxReqNodes>unlimited</maxReqNodes>
	      <maxRunningJobs>unlimited</maxRunningJobs>
	      <maxJobsInQueue>unlimited</maxJobsInQueue>
	      */ 
	}
	
	
	/**
	 * Send XPath query to DefaultIndexService running on host, 
	 * request GLUECE element from resource properties. 
	 */
	public Object getInformation(String resourceURL, int timeout) throws Exception {
		
		String serviceURL = null; 
		
		try {
			
			logger.warn("Query to host: " + resourceURL);
			
			serviceURL = this.translateURL(resourceURL); 
			logger.warn("Getting information from " + serviceURL);
			
			GlobusCredential globusCredential = JobSubmissionUtils.getDelegatedCredential();
			
			QueryResourceProperties_PortType port = QueryUtils.getQueryPort(serviceURL,
				globusCredential); 
			
			// Build query for GLUE RP and send it to service. 
	        QueryResourceProperties_Element request = 
	        	QueryUtils.getQuery("//*[local-name()='GLUECE']");
			QueryResourcePropertiesResponse response = 
				port.queryResourceProperties(request);
	        // System.out.println(AnyHelper.toSingleString(response));
			MessageElement any[] = response.get_any(); 
		    
		    GLUECERPType glueInfo = (GLUECERPType) 
				ObjectDeserializer.toObject(any[0], GLUECERPType.class);
		    	
		    logger.warn("Cluster name: " + glueInfo.getCluster(0).getName()); 
		    // Perform a second query, getting information about the 
		    // queue from the queueProvider 
		    request = QueryUtils.getQuery("//*[local-name()='scheduler']");
	        response =  port.queryResourceProperties(request);
	        // System.out.println(AnyHelper.toSingleString(response));
	        any = response.get_any(); 
	        SchedulerType schedulerInfo = (SchedulerType) 
				ObjectDeserializer.toObject(any[0], SchedulerType.class); 
	        
	        // Need to fix a few values that are correctly set in the scheduler 
	        // object, but not in the gluecerp object. 
	        
	        // totaljobs 
	        // FIXME: SchedulerType is not the same type as 
	        // the one in the response from the DefaultIndexService 
	        // Some versioning problem? 
	        
	        /*	         
	        int totalJobs = ce.getState().getTotalJobs();      		
    		int totalCPUs = Integer.parseInt(ce.getInfo().getTotalCPUs()); 
    		int freeCPUs = ce.getState().getFreeCPUs(); 
	         */
	        
	        // Add some attributes currently only defined in the 
	        // schedulerInfo to appropiate places in glueInfo 
	        updateGlue(glueInfo, schedulerInfo); 
	        
			return glueInfo; 
			
		} catch (Exception e) {
		    logger.error("No information retrieved form " + serviceURL); 
		    throw new Exception("No information retrieved form " 
		    		+ resourceURL); 
		}
	}
}
